----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Author: Ioana Dabacan
-- 
-- Create Date:    22:26:07 March/03/2008 
-- Design Name: 
-- Module Name:    CON3RefComp - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: This is the simple Digilent CON3 Reference Component.
--             
--   Inputs:
--     - ck = clock signal (50MHz)
--     - vecPwInA = Pulse Width value for channel A
--     - vecPwInB = Pulse Width value for channel B
--     - vecPwInC = Pulse Width value for channel C
--     - vecPwInD = Pulse Width value for channel D
--   Outputs:
--     - bitCtrlOutA = Enable pin for Servo A
--     - bitCtrlOutB = Enable pin for Servo B
--     - bitCtrlOutC = Enable pin for Servo C
--     - bitCtrlOutD = Enable pin for Servo D
--   Internal constants:
--     - intPreDiv = Pre division factor. Defines the division factor
--       used to build the internal clock (ckInt).
--       frequency(ckInt) =  freqquency(ckInt)/intPreDiv.
--     - cntPreDiv = counter for the clock division with the intPreDiv factor
--     - cntPwm = counter for the PWM signal generation
--     - vecPwA = Pulse Width internal value for channel A
--     - vecPwB = Pulse Width internal value for channel B
--     - vecPwC = Pulse Width internal value for channel C
--     - vecPwD = Pulse Width internal value for channel D
--     Behavioral description:
--       In order to create the Pulse Modulated signal for each of the 4 
--       channels, a divided internal clock (ckInt) is used, running at a
--       50 KHz. This will provide a resolution of 20us for the Pulse Width
--       Modulated output signals (bitEnOut*).
--       Each of the vecPwIn* input signals specify the number of 20us ckInt
--       periods that represent the bitEnOut* active pulse length. The servos
--       act hazardous when having a duty factor exactly 1 or 0. Therefore the
--       values (bitEnOut*) are forced on 1 or 254 respectively in order to
--       provide consistent control signals.
--       The cntPwm counter counts 511 ckInt periods for one bitEnOut* period,
--       in order to acomplish the servos specifications.                         
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity CON3RefComp is
 Port (    ck : in  STD_LOGIC;          -- system clock (50MHz)
           vecPwInA : in  STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width value (A)
           vecPwInB : in  STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width value (B)
			  vecPwInC : in  STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width value (C)
			  vecPwInD : in  STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width value (D)
           bitCtrlOutA : out  STD_LOGIC;   -- Enable pin for Servo A
			  bitCtrlOutB : out  STD_LOGIC;   -- Enable pin for Servo B
			  bitCtrlOutC : out  STD_LOGIC;   -- Enable pin for Servo C
			  bitCtrlOutD : out  STD_LOGIC);   -- Enable pin for Servo D
           

end CON3RefComp;

architecture Behavioral of CON3RefComp is

  constant intPreDiv: integer := 1000; -- predivision factor
                             
  signal cntPreDiv: integer range 0 to intPreDiv - 1 := 0; --counter for predivider
  signal ckInt: STD_LOGIC; -- Internal clock for PWM
  signal cntPwm: integer range 0 to 511 := 1; -- counter for PWM
  signal vecPwA : STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width internal value (A)
  signal vecPwB : STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width internal value (B)
  signal vecPwC : STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width internal value (C)
  signal vecPwD : STD_LOGIC_VECTOR(7 downto 0); -- Pulse Width internal value (D)
  

begin

  vecPwA <= vecPwInA+'1' when vecPwInA = x"00" else
				vecPwInA;

  vecPwB <= vecPwInB+'1' when vecPwInB = x"00" else	         
				vecPwInB;

  vecPwC <= vecPwInC+'1' when vecPwInC = x"00" else	        
				vecPwInC;

  vecPwD <= vecPwInD+'1' when vecPwInD = x"00" else	        
				vecPwInD;

 preDiv: process(ck)  -- clock predivision
  begin
    if ck'event and ck = '1' then
	   if cntPreDiv = intPreDiv - 1 then
		  cntPreDiv <= 0;
		  ckInt <= '1';
		else
        cntPreDiv <= cntPreDiv + 1;
		  ckInt <= '0';
      end if;
    end if;
  end process;
  
  
 PWM: process(ckInt)  -- PWM counter
  begin
    if ckInt'event and ckInt = '1' then	
	     cntPwm <= cntPwm + 1;
      end if;
	  end process;	 
  
  bitCtrlOutA <= '1' when cntPwm < vecPwA else  -- enable output for channel A
               '0';
  bitCtrlOutB <= '1' when cntPwm < vecPwB else  -- enable output for channel B
               '0';
  bitCtrlOutC <= '1' when cntPwm < vecPwC else  -- enable output for channel C
               '0';
  bitCtrlOutD <= '1' when cntPwm < vecPwD else  -- enable output for channel D
               '0';


end Behavioral;

